import { of } from 'rxjs';
import { map, mergeMap, switchMap } from 'rxjs/operators';
import { Err, Ok } from './index';
export function resultMap(mapper, errMapper) {
    return (source) => {
        return source.pipe(map(result => {
            if (result.ok) {
                if (mapper) {
                    const newVal = mapper(result.val);
                    return new Ok(newVal);
                }
                else {
                    return result;
                }
            }
            else {
                if (errMapper) {
                    return new Err(errMapper(result.val));
                }
                else {
                    return result;
                }
            }
        }));
    };
}
export function resultMapTo(value) {
    return (source) => {
        return source.pipe(map(result => {
            if (result.ok) {
                return new Ok(value);
            }
            else {
                return result;
            }
        }));
    };
}
export function elseMap(mapper) {
    return (source) => {
        return source.pipe(map(result => {
            if (result.err) {
                return mapper(result.val);
            }
            else {
                return result.val;
            }
        }));
    };
}
export function elseMapTo(value) {
    return (source) => {
        return source.pipe(map(result => {
            if (result.err) {
                return value;
            }
            else {
                return result.val;
            }
        }));
    };
}
export function resultSwitchMap(mapper) {
    return (source) => {
        return source.pipe(switchMap(result => {
            if (result.ok) {
                return mapper(result.val);
            }
            else {
                return of(result);
            }
        }), map(result => {
            if (result instanceof Ok || result instanceof Err) {
                return result;
            }
            else {
                return new Ok(result);
            }
        }));
    };
}
export function resultMergeMap(mapper) {
    return (source) => {
        return source.pipe(mergeMap(result => {
            if (result.ok) {
                return mapper(result.val);
            }
            else {
                return of(result);
            }
        }), map(result => {
            if (result instanceof Ok || result instanceof Err) {
                return result;
            }
            else {
                return new Ok(result);
            }
        }));
    };
}
//# sourceMappingURL=rxjs-operators.js.map